@@ -3,7 +3,7 @@
 # This file defines the default bindings for Tk entry widgets and provides
 # procedures that help in implementing those bindings.
 #
-# SCCS: @(#) entry.tcl 1.49 97/09/17 19:08:48
+# RCS: @(#) $Id: entry.tcl,v 1.7 1999/08/09 16:52:06 hobbs Exp $
 #
 # Copyright (c) 1992-1994 The Regents of the University of California.
 # Copyright (c) 1994-1997 Sun Microsystems, Inc.
@@ -26,30 +26,31 @@
 #			char, word, or line.
 # x, y -		Last known mouse coordinates for scanning
 #			and auto-scanning.
+# data -		Used for Cut and Copy
 #-------------------------------------------------------------------------
 
 #-------------------------------------------------------------------------
 # The code below creates the default class bindings for entries.
 #-------------------------------------------------------------------------
 bind Entry <<Cut>> {
-    if {![catch {set data [string range [%W get] [%W index sel.first]\
-	    [expr {[%W index sel.last] - 1}]]}]} {
+    if {![catch {tkEntryGetSelection %W} tkPriv(data)]} {
 	clipboard clear -displayof %W
-	clipboard append -displayof %W $data
+	clipboard append -displayof %W $tkPriv(data)
 	%W delete sel.first sel.last
+	unset tkPriv(data)
     }
 }
 bind Entry <<Copy>> {
-    if {![catch {set data [string range [%W get] [%W index sel.first]\
-	    [expr {[%W index sel.last] - 1}]]}]} {
+    if {![catch {tkEntryGetSelection %W} tkPriv(data)]} {
 	clipboard clear -displayof %W
-	clipboard append -displayof %W $data
+	clipboard append -displayof %W $tkPriv(data)
+	unset tkPriv(data)
     }
 }
 bind Entry <<Paste>> {
     global tcl_platform
     catch {
-	if {"$tcl_platform(platform)" != "unix"} {
+      if {[string compare $tcl_platform(platform) "unix"]} {
 	    catch {
 		%W delete sel.first sel.last
 	    }
@@ -201,13 +202,13 @@ bind Entry <Escape> {# nothing}
 bind Entry <Return> {# nothing}
 bind Entry <KP_Enter> {# nothing}
 bind Entry <Tab> {# nothing}
-if {$tcl_platform(platform) == "macintosh"} {
+if {![string compare $tcl_platform(platform) "macintosh"]} {
 	bind Entry <Command-KeyPress> {# nothing}
 }
 
 # On Windows, paste is done using Shift-Insert.  Shift-Insert already
 # generates the <<Paste>> event, so we don't need to do anything here.
-if {$tcl_platform(platform) != "windows"} {
+if {[string compare $tcl_platform(platform) "windows"]} {
     bind Entry <Insert> {
 	catch {tkEntryInsert %W [selection get -displayof %W]}
     }
@@ -335,7 +336,7 @@ proc tkEntryButton1 {w x} {
     set tkPriv(pressX) $x
     $w icursor [tkEntryClosestGap $w $x]
     $w selection from insert
-    if {[lindex [$w configure -state] 4] == "normal"} {focus $w}
+    if {![string compare [$w cget -state] "normal"]} {focus $w}
 }
 
 # tkEntryMouseSelect --
@@ -405,7 +406,7 @@ proc tkEntryPaste {w x} {
 
     $w icursor [tkEntryClosestGap $w $x]
     catch {$w insert insert [selection get -displayof $w]}
-    if {[lindex [$w configure -state] 4] == "normal"} {focus $w}
+    if {![string compare [$w cget -state] "normal"]} {focus $w}
 }
 
 # tkEntryAutoScan --
@@ -462,7 +463,7 @@ proc tkEntryKeySelect {w new} {
 # s -		The string to insert (usually just a single character)
 
 proc tkEntryInsert {w s} {
-    if {$s == ""} {
+    if {![string compare $s ""]} {
 	return
     }
     catch {
@@ -535,98 +536,6 @@ proc tkEntrySetCursor {w pos} {
     tkEntrySeeInsert $w
 }
 
-# tkValidate --
-# This procedure validates particular types of numbers/formats
-#
-# Arguments:
-# type		- The type of validation (alphabetic, alphanumeric, date,
-#		hex, integer, numeric, real).  Date is always strict.
-# val		- The value to be validated
-#
-# Switches:
-# -strict	- enable more precise (strict) pattern matching on number
-#
-# Example use:	tkValidate real 55e-5
-#		tkValidate -strict integer -505
-#
-
-proc tkValidate {args} {
-  set strict 0
-  if [string match "-s*" [lindex $args 0]] {
-    set strict 1
-    set args [lreplace $args 0 0]
-  }
-
-  if {[llength $args] == 2} {
-    set type [lindex $args 0]
-    set val  [lindex $args 1]
-  } else {
-    error "called \"tkValidate\" with improper arguments\
-	\nmust be 'tkValidate ?-strict? type value'"    
-  }
-
-  switch -glob -- $type {
-    alphab* {
-      #alphabetic
-      if {$strict} {
-	return [regexp -nocase {^[a-z]+$} $val]
-      } else {
-	return [regexp -nocase {^[a-z]*$} $val]
-      }
-    }
-    alphan* {
-      #alphanumeric
-      if {$strict} {
-	return [regexp -nocase {^[a-z0-9]+$} $val]
-      } else {
-	return [regexp -nocase {^[a-z0-9]*$} $val]
-      }
-    }
-    d* {
-      #date
-      return [expr ![catch {clock scan $val}]]
-    }
-    h* {
-      #hexadecimal
-      if {$strict} {
-	return -nocase [regexp {^(0x)?[0-9a-f]+$} $val]
-      } else {
-	return -nocase [regexp {^(0x)?[0-9a-f]*$} $val]
-      }
-    }
-    i* {
-      #integer
-      if {$strict} {
-	return [regexp {^[-+]?[0-9]+$} $val]
-      } else {
-	return [regexp {^[-+]?[0-9]*$} $val]
-      }
-    }
-    n* {
-      #numeric
-      if {$strict} {
-	return [regexp {^[0-9]+$} $val]
-      } else {
-	return [regexp {^[0-9]*$} $val]
-      }
-    }
-    r* {
-      #real
-      if {$strict} {
-	return [regexp -nocase \
-	    {^[-+]?([0-9]+\.?[0-9]*|[0-9]*\.?[0-9]+)(e[-+]?[0-9]+)?$} $val]
-      } else {
-	return [regexp -nocase \
-	    {^[-+]?[0-9]*\.?[0-9]*([0-9]\.?e[-+]?[0-9]*)?$} $val]
-      }
-    }
-    default {
-      error "called \"tkValidate\" with unknown type \"$type\":\
-	\nmust be alphabetic, alphanumeric, date, hexadecimal, integer, numeric or real"
-    }
-  }
-}
-
 # tkEntryTranspose -
 # This procedure implements the "transpose" function for entry widgets.
 # It tranposes the characters on either side of the insertion cursor,
@@ -662,7 +571,7 @@ proc tkEntryTranspose w {
 # w -		The entry window in which the cursor is to move.
 # start -	Position at which to start search.
 
-if {$tcl_platform(platform) == "windows"}  {
+if {![string compare $tcl_platform(platform) "windows"]}  {
     proc tkEntryNextWord {w start} {
 	set pos [tcl_endOfWord [$w get] [$w index $start]]
 	if {$pos >= 0} {
@@ -699,4 +608,18 @@ proc tkEntryPreviousWord {w start} {
     }
     return $pos
 }
+# tkEntryGetSelection --
+#
+# Returns the selected text of the entry with respect to the -show option.
+#
+# Arguments:
+# w -         The entry window from which the text to get
 
+proc tkEntryGetSelection {w} {
+    set entryString [string range [$w get] [$w index sel.first] \
+                       [expr [$w index sel.last] - 1]]
+    if {[$w cget -show] != ""} {
+      regsub -all . $entryString [string index [$w cget -show] 0] entryString
+    }
+    return $entryString
+}
