@@ -3,12 +3,12 @@
  *
  *	Implements common window manager functions for the Macintosh.
  *
- * Copyright (c) 1995-1997 Sun Microsystems, Inc.
+ * Copyright (c) 1995-1998 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkMacWindowMgr.c 1.59 97/11/20 18:56:39
+ * RCS: @(#) $Id: tkMacWindowMgr.c,v 1.4 1999/04/16 01:51:32 stanton Exp $
  */
 
 #include <Events.h>
@@ -63,13 +63,14 @@ static int 	GenerateActivateEvents _ANSI_ARGS_((EventRecord *eventPtr,
 static int 	GenerateFocusEvent _ANSI_ARGS_((EventRecord *eventPtr,
 			Window window));
 static int	GenerateKeyEvent _ANSI_ARGS_((EventRecord *eventPtr,
-			Window window));
+			Window window, UInt32 savedCode));
 static int	GenerateUpdateEvent _ANSI_ARGS_((EventRecord *eventPtr,
 			Window window));
 static void 	GenerateUpdates _ANSI_ARGS_((RgnHandle updateRgn,
 			TkWindow *winPtr));
 static int 	GeneratePollingEvents _ANSI_ARGS_((void));	
-static int 	GeneratePollingEvents2 _ANSI_ARGS_((Window window));	
+static int 	GeneratePollingEvents2 _ANSI_ARGS_((Window window,
+	                int adjustCursor));	
 static OSErr	TellWindowDefProcToCalcRegions _ANSI_ARGS_((WindowRef wRef));
 static int	WindowManagerMouse _ANSI_ARGS_((EventRecord *theEvent,
 		    Window window));
@@ -103,6 +104,7 @@ WindowManagerMouse(
     Point where, where2;
     int xOffset, yOffset;
     short windowPart;
+    TkDisplay *dispPtr;
 				
     frontWindow = FrontWindow();
 
@@ -121,7 +123,8 @@ WindowManagerMouse(
     }
 
     windowPart = FindWindow(eventPtr->where, &whichWindow);
-    tkwin = Tk_IdToWindow(tkDisplayList->display, window);
+    dispPtr = TkGetDisplayList();
+    tkwin = Tk_IdToWindow(dispPtr->display, window);
     switch (windowPart) {
 	case inSysWindow:
 	    SystemClick(eventPtr, (GrafPort *) whichWindow);
@@ -292,8 +295,10 @@ GenerateUpdateEvent(
 {
     WindowRef macWindow;
     register TkWindow *winPtr;
+    TkDisplay *dispPtr;
 	
-    winPtr = (TkWindow *) Tk_IdToWindow(tkDisplayList->display, window);
+    dispPtr = TkGetDisplayList();
+    winPtr = (TkWindow *) Tk_IdToWindow(dispPtr->display, window);
 
     if (winPtr == NULL) {
 	 return false;
@@ -463,6 +468,7 @@ TkGenerateButtonEvent(
     Point where;
     Tk_Window tkwin;
     int dummy;
+    TkDisplay *dispPtr;
 
     /* 
      * ButtonDown events will always occur in the front
@@ -479,7 +485,8 @@ TkGenerateButtonEvent(
 	return false;
     }
 
-    tkwin = Tk_IdToWindow(tkDisplayList->display, window);
+    dispPtr = TkGetDisplayList();
+    tkwin = Tk_IdToWindow(dispPtr->display, window);
     
     GlobalToLocal(&where);
     if (tkwin != NULL) {
@@ -516,8 +523,10 @@ GenerateActivateEvents(
     Window window)		/* Root X window for event. */
 {
     TkWindow *winPtr;
+    TkDisplay *dispPtr;
     
-    winPtr = (TkWindow *) Tk_IdToWindow(tkDisplayList->display, window);
+    dispPtr = TkGetDisplayList();
+    winPtr = (TkWindow *) Tk_IdToWindow(dispPtr->display, window);
     if (winPtr == NULL || winPtr->window == None) {
 	return false;
     }
@@ -628,8 +637,10 @@ GenerateFocusEvent(
 {
     XEvent event;
     Tk_Window tkwin;
+    TkDisplay *dispPtr;
     
-    tkwin = Tk_IdToWindow(tkDisplayList->display, window);
+    dispPtr = TkGetDisplayList();
+    tkwin = Tk_IdToWindow(dispPtr->display, window);
     if (tkwin == NULL) {
 	return false;
     }
@@ -645,9 +656,9 @@ GenerateFocusEvent(
 	event.xany.type = FocusOut;
     }
 
-    event.xany.serial = tkDisplayList->display->request;
+    event.xany.serial = dispPtr->display->request;
     event.xany.send_event = False;
-    event.xfocus.display = tkDisplayList->display;
+    event.xfocus.display = dispPtr->display;
     event.xfocus.window = window;
     event.xfocus.mode = NotifyNormal;
     event.xfocus.detail = NotifyDetailNone;
@@ -678,23 +689,42 @@ GenerateFocusEvent(
 static int
 GenerateKeyEvent(
     EventRecord *eventPtr,	/* Incoming Mac event */
-    Window window)		/* Root X window for event. */
+    Window window,		/* Root X window for event. */
+    UInt32 savedKeyCode)	/* If non-zero, this is a lead byte which
+    				 * should be combined with the character
+    				 * in this event to form one multi-byte 
+    				 * character. */
 {
     Point where;
     Tk_Window tkwin;
     XEvent event;
-
+    unsigned char byte;
+    char buf[16];
+    TkDisplay *dispPtr;
+    
     /*
      * The focus must be in the FrontWindow on the Macintosh.
      * We then query Tk to determine the exact Tk window
      * that owns the focus.
      */
 
-    tkwin = Tk_IdToWindow(tkDisplayList->display, window);
+    dispPtr = TkGetDisplayList();
+    tkwin = Tk_IdToWindow(dispPtr->display, window);
     tkwin = (Tk_Window) ((TkWindow *) tkwin)->dispPtr->focusPtr;
     if (tkwin == NULL) {
 	return false;
     }
+    byte = (unsigned char) (eventPtr->message & charCodeMask);
+    if ((savedKeyCode == 0) && 
+            (Tcl_ExternalToUtf(NULL, NULL, (char *) &byte, 1, 0, NULL, 
+            	    buf, sizeof(buf), NULL, NULL, NULL) != TCL_OK)) {
+        /*
+         * This event specifies a lead byte.  Wait for the second byte
+         * to come in before sending the XEvent.
+         */
+         
+        return false;
+    }   
 
     where.v = eventPtr->where.v;
     where.h = eventPtr->where.h;
@@ -709,7 +739,10 @@ GenerateKeyEvent(
     GlobalToLocal(&where);
     Tk_TopCoordsToWindow(tkwin, where.h, where.v, 
 	    &event.xkey.x, &event.xkey.y);
-    event.xkey.keycode = eventPtr->message;
+    
+    event.xkey.keycode = byte |
+            ((savedKeyCode & charCodeMask) << 8) |
+            ((eventPtr->message & keyCodeMask) << 8);
 
     event.xany.serial = Tk_Display(tkwin)->request;
     event.xkey.window = Tk_WindowId(tkwin);
@@ -769,7 +802,8 @@ GeneratePollingEvents()
     short part;
     int local_x, local_y;
     int generatedEvents = false;
-    
+    TkDisplay *dispPtr;
+
     /*
      * First we get the current mouse position and determine
      * what Tk window the mouse is over (if any).
@@ -791,7 +825,8 @@ GeneratePollingEvents()
 	tkwin = NULL;
     } else {
 	window = TkMacGetXWindow(whichwindow);
-	rootwin = Tk_IdToWindow(tkDisplayList->display, window);
+	dispPtr = TkGetDisplayList();
+	rootwin = Tk_IdToWindow(dispPtr->display, window);
 	if (rootwin == NULL) {
 	    tkwin = NULL;
 	} else {
@@ -810,7 +845,7 @@ GeneratePollingEvents()
     }
     Tk_UpdatePointer(tkwin, whereGlobal.h,  whereGlobal.v,
 	    TkMacButtonKeyState());
-
+    
     /*
      * Finally, we make sure the proper cursor is installed.  The installation
      * is polled to 1) make our resize hack work, and 2) make sure we have the 
@@ -849,7 +884,8 @@ GeneratePollingEvents()
 
 static int
 GeneratePollingEvents2(
-    Window window)
+    Window window,
+    int adjustCursor)
 {
     Tk_Window tkwin, rootwin;
     WindowRef whichwindow, frontWin;
@@ -857,6 +893,7 @@ GeneratePollingEvents2(
     int local_x, local_y;
     int generatedEvents = false;
     Rect bounds;
+    TkDisplay *dispPtr;
     
     /*
      * First we get the current mouse position and determine
@@ -879,7 +916,8 @@ GeneratePollingEvents2(
     if (whichwindow != frontWin) {
 	tkwin = NULL;
     } else {
-	rootwin = Tk_IdToWindow(tkDisplayList->display, window);
+        dispPtr = TkGetDisplayList();
+	rootwin = Tk_IdToWindow(dispPtr->display, window);
 	TkMacWinBounds((TkWindow *) rootwin, &bounds);
 	if (!PtInRect(whereLocal, &bounds)) {
 	    tkwin = NULL;
@@ -889,6 +927,7 @@ GeneratePollingEvents2(
 	}
     }
 
+    
     /*
      * The following call will generate the appropiate X events and
      * adjust any state that Tk must remember.
@@ -899,15 +938,17 @@ GeneratePollingEvents2(
     }
     Tk_UpdatePointer(tkwin, whereGlobal.h,  whereGlobal.v,
 	    TkMacButtonKeyState());
-
+    
     /*
      * Finally, we make sure the proper cursor is installed.  The installation
      * is polled to 1) make our resize hack work, and 2) make sure we have the 
      * proper cursor even if someone else changed the cursor out from under
      * us.
      */
-    TkMacInstallCursor(0);
-
+     
+    if (adjustCursor) {
+        TkMacInstallCursor(0);
+    }
     return true;
 }
 
@@ -1105,6 +1146,7 @@ TkMacConvertEvent(
     WindowRef whichWindow;
     Window window;
     int eventFound = false;
+    static UInt32 savedKeyCode;
     
     switch (eventPtr->what) {
 	case nullEvent:
@@ -1148,11 +1190,28 @@ TkMacConvertEvent(
 		    break;
 		}
 	    }
+	    /* fall through */
+	    
 	case keyUp:
 	    whichWindow = FrontWindow();
+	    if (whichWindow == NULL) {
+	        /*
+	         * This happens if we get a key event before Tk has had a
+	         * chance to actually create and realize ".", if they type
+	         * when "." is withdrawn(!), or between the time "." is 
+	         * destroyed and the app exits.
+	         */
+	         
+	        return false;
+	    }
 	    window = TkMacGetXWindow(whichWindow);
-	    eventFound |= GenerateKeyEvent(eventPtr, window);
+	    if (GenerateKeyEvent(eventPtr, window, savedKeyCode) == 0) {
+	        savedKeyCode = eventPtr->message;
+	        return false;
+	    }
+	    eventFound = true;
 	    break;
+	    	    
 	case activateEvt:
 	    window = TkMacGetXWindow((WindowRef) eventPtr->message);
 	    eventFound |= GenerateActivateEvents(eventPtr, window);
@@ -1205,6 +1264,7 @@ TkMacConvertEvent(
 	    break;
     }
     
+    savedKeyCode = 0;
     return eventFound;
 }
 
@@ -1214,7 +1274,7 @@ TkMacConvertEvent(
  * TkMacConvertTkEvent --
  *
  *	This function converts a Macintosh event into zero or more
- *	Tcl events.
+ *	Tcl events.  It is intended for use in Netscape-style embedding.
  *
  * Results:
  *	Returns 1 if event added to Tcl queue, 0 otherwse.
@@ -1232,15 +1292,36 @@ TkMacConvertTkEvent(
 {
     int eventFound = false;
     Point where;
+    static UInt32 savedKeyCode;
+    
+    /*
+     * By default, assume it is legal for us to set the cursor 
+     */
+     
+    Tk_MacTkOwnsCursor(1);
     
     switch (eventPtr->what) {
 	case nullEvent:
+        /*
+         * We get NULL events only when the cursor is NOT over
+	 * the plugin.  Otherwise we get updateCursor events.
+	 * We will not generate polling events or move the cursor
+	 * in this case.
+         */
+            
+	    eventFound = false;
+	    break;
 	case adjustCursorEvent:
-	    if (GeneratePollingEvents2(window)) {
+	    if (GeneratePollingEvents2(window, 1)) {
 		eventFound = true;
 	    }
 	    break;
 	case updateEvt:
+        /*
+         * It is possibly not legal for us to set the cursor 
+         */
+     
+            Tk_MacTkOwnsCursor(0);
 	    if (GenerateUpdateEvent(eventPtr, window)) {
 		eventFound = true;
 	    }
@@ -1267,10 +1348,24 @@ TkMacConvertTkEvent(
 		    break;
 		}
 	    }
+	    /* fall through. */
+	    
 	case keyUp:
-	    eventFound |= GenerateKeyEvent(eventPtr, window);
+	    if (GenerateKeyEvent(eventPtr, window, savedKeyCode) == 0) {
+	        savedKeyCode = eventPtr->message;
+	        return false;
+	    }	        
+	    eventFound = true;
 	    break;
+	    
 	case activateEvt:
+        /*
+         * It is probably not legal for us to set the cursor
+	 * here, since we don't know where the mouse is in the
+	 * window that is being activated.
+         */
+     
+            Tk_MacTkOwnsCursor(0);
 	    eventFound |= GenerateActivateEvents(eventPtr, window);
 	    eventFound |= GenerateFocusEvent(eventPtr, window);
 	    break;
@@ -1291,10 +1386,18 @@ TkMacConvertTkEvent(
 	     * Do clipboard conversion.
 	     */
 	    switch ((eventPtr->message & osEvtMessageMask) >> 24) {
+        /*
+         * It is possibly not legal for us to set the cursor.
+         * Netscape sends us these events all the time... 
+         */
+     
+                Tk_MacTkOwnsCursor(0);
+        
 		case mouseMovedMessage:
-		    if (GeneratePollingEvents2(window)) {
+		    /* if (GeneratePollingEvents2(window, 0)) {
 			eventFound = true;
-		    }
+		    }  NEXT LINE IS TEMPORARY */
+		    eventFound = false;
 		    break;
 		case suspendResumeMessage:
 		    if (!(eventPtr->message & resumeFlag)) {
@@ -1318,7 +1421,7 @@ TkMacConvertTkEvent(
 	    }
 	    break;
     }
-    
+    savedKeyCode = 0;    
     return eventFound;
 }
 
@@ -1516,7 +1619,6 @@ TellWindowDefProcToCalcRegions(
      * Assuming there are no errors we now call the window definition 
      * procedure to tell it to calculate the regions for the window.
      */
-
     if (err == noErr) {
  	(void) CallWindowDefProc((UniversalProcPtr) *wdef,
 		GetWVariant(wRef), wRef, wCalcRgns, 0);
